@@ -29,13 +29,14 @@ from merlin.models.tf.inputs.base import InputBlockV2
 from merlin.models.tf.inputs.embedding import CombinerType, EmbeddingTable
 from merlin.models.tf.models.base import BaseModel, get_output_schema
 from merlin.models.tf.outputs.topk import TopKOutput
+from merlin.models.tf.transforms.tensor import PrepareFeatures
 from merlin.models.tf.utils import tf_utils
 from merlin.schema import ColumnSchema, Schema, Tags
 
 
 @tf.keras.utils.register_keras_serializable(package="merlin.models")
 class Encoder(tf.keras.Model):
-    """Block that can be used for prediction & evaluation but not for training
+    """Block that can be used for prediction and evaluation but not for training
 
     Parameters
     ----------
@@ -72,6 +73,7 @@ class Encoder(tf.keras.Model):
         self.blocks = [input_block] + list(blocks) if blocks else [input_block]
         self.pre = pre
         self.post = post
+        self.prepare_features = PrepareFeatures(self._schema)
 
     def encode(
         self,
@@ -161,21 +163,13 @@ class Encoder(tf.keras.Model):
         return combinators.call_sequentially(
             list(self.to_call),
             inputs=inputs,
-            features=inputs,
+            features=self.prepare_features(inputs),
             targets=targets,
             training=training,
             testing=testing,
             **kwargs,
         )
 
-    def build(self, input_shape):
-        combinators.build_sequentially(self, list(self.to_call), input_shape=input_shape)
-        if not hasattr(self.build, "_is_default"):
-            self._build_input_shape = input_shape
-
-    def compute_output_shape(self, input_shape):
-        return combinators.compute_output_shape_sequentially(list(self.to_call), input_shape)
-
     def __call__(self, inputs, **kwargs):
         # We remove features here since we don't expect them at inference time
         # Inside the `call` method, we will add them back by assuming inputs=features
@@ -184,6 +178,14 @@ class Encoder(tf.keras.Model):
 
         return super().__call__(inputs, **kwargs)
 
+    def build(self, input_shape):
+        combinators.build_sequentially(self, list(self.to_call), input_shape=input_shape)
+        if not hasattr(self.build, "_is_default"):
+            self._build_input_shape = input_shape
+
+    def compute_output_shape(self, input_shape):
+        return combinators.compute_output_shape_sequentially(list(self.to_call), input_shape)
+
     def train_step(self, data):
         """Train step"""
         raise NotImplementedError(
