@@ -198,8 +198,6 @@ class Loader(merlin.dataloader.tensorflow.Loader):
         `reader_kwargs` will be ignored
     batch_size: int
         Number of samples to yield at each iteration
-    transform: Union[Callable, SchemaAwareTransform], optional
-        Transformation to apply to each batch of data.
     label_names: list(str)
         Column name of the target variable in the dataframe specified by
         `paths_or_dataset`
@@ -250,7 +248,6 @@ class Loader(merlin.dataloader.tensorflow.Loader):
         self,
         paths_or_dataset,
         batch_size,
-        transform=None,
         label_names=None,
         feature_columns=None,
         cat_names=None,
@@ -267,7 +264,6 @@ class Loader(merlin.dataloader.tensorflow.Loader):
         drop_last=False,
         sparse_names=None,
         sparse_max=None,
-        multi_label_as_dict=True,
         sparse_as_dense=False,
         schema=None,
     ):
@@ -334,21 +330,31 @@ class Loader(merlin.dataloader.tensorflow.Loader):
         self.sparse_max = sparse_max or {}
         self.sparse_as_dense = sparse_as_dense
 
-    @property
-    def input_schema(self) -> Schema:
-        return self.dataset.schema
-
     @property
     def output_schema(self) -> Schema:
-        schema = self.input_schema
-
+        output_schema = super().output_schema
         for map_fn in self._map_fns:
             if hasattr(map_fn, "compute_output_schema"):
-                schema = map_fn.compute_output_schema(schema)
+                output_schema = map_fn.compute_output_schema(output_schema)
             else:
-                raise ValueError(f"Couldn't infer schema from transform {map_fn}")
+                raise ValueError(
+                    f"Couldn't infer schema from transform {map_fn}. "
+                    "Please implement the `compute_output_schema` method on "
+                    "the transform layer."
+                )
 
-        return schema
+        return output_schema
+
+    @property
+    def has_transforms(self) -> bool:
+        """Returns True if Loader has transforms or map functions.
+
+        Returns
+        -------
+        bool
+            True if Loader has transforms or map functions, otherwise False.
+        """
+        return len(self._map_fns) > 0 or self.transforms is not None
 
 
 KerasSequenceValidater = (
@@ -363,7 +369,7 @@ def sample_batch(
     include_targets: bool = True,
     to_ragged: bool = False,
     to_dense: bool = False,
-    process_lists=False,
+    prepare_features=False,
 ):
     """Util function to generate a batch of input tensors from a merlin.io.Dataset instance
 
@@ -381,6 +387,11 @@ def sample_batch(
         Whether to convert the tuple of sparse tensors into ragged tensors, by default False.
     to_dense: bool
         Whether to convert the tuple of sparse tensors into dense tensors, by default False.
+    prepare_features: bool
+        Whether to prepare features from dataloader for the model, by default False.
+        If enabled, it converts multi-hot/list features to dense or ragged based on the schema.
+        It also ensures that scalar features are converted to 2D (batch size, 1).
+        P.s. The features are automatically prepared by InputBlockV2 if it is used
     Returns:
     -------
     batch: Dict[tf.tensor]
@@ -391,7 +402,7 @@ def sample_batch(
             "Sparse values cannot be converted to both ragged tensors and dense tensors"
         )
 
-    from merlin.models.tf.transforms.tensor import ListToDense, ListToRagged, ProcessList
+    from merlin.models.tf.transforms.tensor import ListToDense, ListToRagged, PrepareFeatures
 
     if isinstance(dataset_or_loader, Dataset):
         if not batch_size:
@@ -408,8 +419,8 @@ def sample_batch(
         inputs = ListToRagged()(inputs)
     elif to_dense:
         inputs = ListToDense()(inputs)
-    if process_lists:
-        inputs = ProcessList(loader.schema)(inputs)
+    if prepare_features:
+        inputs = PrepareFeatures(loader.schema)(inputs)
     if not include_targets:
         return inputs
     return inputs, targets
