@@ -255,10 +255,11 @@ class TopkMetric(Mean, TopkMetricWithLabelRelevantCountsMixin):
         y_pred: tf.Tensor,
         sample_weight: Optional[tf.Tensor] = None,
     ):
+        y_true, y_pred = tf.squeeze(y_true), tf.squeeze(y_pred)
         tf.debugging.assert_greater_equal(
             tf.shape(y_true)[-1],
             self.k,
-            f"The TopkMetric {self.name} cutoff ({self.k}) cannot be smaller than "
+            f"The TopkMetric {self.name} cutoff ({self.k}) cannot be bigger than "
             f"the number of predictions per example",
         )
 
@@ -354,14 +355,14 @@ class TopkMetric(Mean, TopkMetricWithLabelRelevantCountsMixin):
 
 
 @tf.keras.utils.register_keras_serializable(package="merlin.models")
-@metrics_registry.register_with_multiple_names("recall_at", "recall")
+@metrics_registry.register_with_multiple_names("recall_at")
 class RecallAt(TopkMetric):
     def __init__(self, k=10, pre_sorted=False, name="recall_at", **kwargs):
         super().__init__(recall_at, k=k, pre_sorted=pre_sorted, name=name, **kwargs)
 
 
 @tf.keras.utils.register_keras_serializable(package="merlin.models")
-@metrics_registry.register_with_multiple_names("precision_at", "precision")
+@metrics_registry.register_with_multiple_names("precision_at")
 class PrecisionAt(TopkMetric):
     def __init__(self, k=10, pre_sorted=False, name="precision_at", **kwargs):
         super().__init__(precision_at, k=k, pre_sorted=pre_sorted, name=name, **kwargs)
@@ -417,6 +418,8 @@ class TopKMetricsAggregator(Metric, TopkMetricWithLabelRelevantCountsMixin):
     def update_state(
         self, y_true: tf.Tensor, y_pred: tf.Tensor, sample_weight: Optional[tf.Tensor] = None
     ):
+        # squeeze dim=1
+        y_true, y_pred = tf.squeeze(y_true), tf.squeeze(y_pred)
         # For prediction tensor with rank > 2 (e.g. sequences)
         # reshapes the predictions, targets and label_relevant_counts
         # so that they are 2D and extract_topk() work properly
@@ -457,8 +460,12 @@ class TopKMetricsAggregator(Metric, TopkMetricWithLabelRelevantCountsMixin):
 
     def result(self):
         outputs = {}
+        weighted = "weighted_" in self._name
         for metric in self.topk_metrics:
-            outputs[metric.name] = metric.result()
+            name = metric.name
+            if weighted:
+                name = "weighted_" + name
+            outputs[name] = metric.result()
 
         return outputs
 
@@ -543,6 +550,9 @@ def split_metrics(
     """
     topk_metrics, topk_aggregators, other_metrics = [], [], []
     for metric in metrics:
+        if isinstance(metric, str):
+            metric = metrics_registry.parse(metric)
+
         if isinstance(metric, TopkMetric):
             topk_metrics.append(metric)
         elif isinstance(metric, TopKMetricsAggregator):
