@@ -342,7 +342,7 @@ def MatrixFactorizationModelV2(
     query_id_tag : Tag
         The tag to select query-id feature, by default `Tags.USER_ID`
     candidate_id_tag : Tag
-        The tag to select cnadidate-id feature, by default `Tags.ITEM_ID`
+        The tag to select candidate-id feature, by default `Tags.ITEM_ID`
     embeddings_initializers : Optional[Dict[str, Callable[[Any], None]]] = None
         An initializer function or a dict where keys are feature names and values are
         callable to initialize embedding tables
@@ -416,6 +416,7 @@ def TwoTowerModelV2(
     outputs: Optional[Union[ModelOutput, List[ModelOutput]]] = None,
     logits_temperature: float = 1.0,
     negative_samplers: ItemSamplersType = None,
+    schema: Schema = None,
     **kwargs,
 ) -> RetrievalModelV2:
     """Builds the Two-tower architecture, as proposed in [1].
@@ -450,6 +451,8 @@ def TwoTowerModelV2(
         List of samplers for negative sampling, by default None
         If the `outputs` and `negative_samplers` are not specified the two tower model
         is trained with contrastive learning and `in-batch` negative sampling strategy.
+    schema: Schema
+        A schema with all input features fed to the two-tower model.
 
     Returns
     -------
@@ -480,6 +483,7 @@ def TwoTowerModelV2(
         query=query_tower,
         candidate=candidate_tower,
         output=outputs,
+        schema=schema,
     )
 
     return model
@@ -529,7 +533,7 @@ def YoutubeDNNRetrievalModelV2(
     schema: Schema
         The `Schema` with the input features
     candidate_id_tag : Tag
-        The tag to select cnadidate-id feature, by default `Tags.ITEM_ID`
+        The tag to select candidate-id feature, by default `Tags.ITEM_ID`
     top_block: tf.keras.layers.Layer
         The hidden layers to apply on top of the features representation
         vector.
