@@ -20,7 +20,6 @@ from tensorflow.keras.backend import random_bernoulli
 
 from merlin.models.tf.core.base import Block, BlockType, PredictionOutput
 from merlin.models.tf.core.combinators import TabularBlock
-from merlin.models.tf.core.prediction import Prediction
 from merlin.models.tf.transforms.tensor import ListToRagged
 from merlin.models.tf.typing import TabularData
 from merlin.models.tf.utils import tf_utils
@@ -76,9 +75,8 @@ class RemovePad3D(Block):
 
 @tf.keras.utils.register_keras_serializable(package="merlin_models")
 class SequenceTransform(TabularBlock):
-    """Prepares sequential inputs and targets for next-item prediction.
-    The target is extracted from the shifted sequence of item ids and
-    the sequential input features are truncated in the last position.
+    """Base class for preparing sequential inputs and targets.
+
     Parameters
     ----------
     schema : Schema
@@ -140,12 +138,12 @@ class SequenceTransform(TabularBlock):
             )
 
         target_shape = inputs[self.target_name].get_shape().as_list()
-        if len(target_shape) != 2:
+        if len(target_shape) < 2:
             raise ValueError(
-                f"The target column ({self.target_name}) is expected to be a 2D tensor,"
+                f"The sequential target column ({self.target_name}) cannot be a 1D tensor,"
                 f" but the shape is {target_shape}"
             )
-        if target_shape[-1] == 1:
+        if target_shape[1] == 1:
             raise ValueError(
                 f"The 2nd dim of the target column ({self.target_name}) should be greater"
                 " than 1, so that the sequential input can be shifted as target"
@@ -171,8 +169,7 @@ class SequenceTransform(TabularBlock):
                 if isinstance(v, tuple) and isinstance(v[1], tf.TensorShape):
                     new_input_shapes[k] = tf.TensorShape([v[1][0], None])
                 else:
-                    # Reducing 1 position of the seq length
-                    new_input_shapes[k] = tf.TensorShape([v[0], v[1] - 1])
+                    new_input_shapes[k] = v
 
         return new_input_shapes
 
@@ -237,6 +234,20 @@ class SequencePredictNext(SequenceTransform):
 
         return (new_inputs, targets)
 
+    def compute_output_shape(self, input_shape):
+        new_input_shapes = dict()
+        for k, v in input_shape.items():
+            new_input_shapes[k] = v
+            if k in self.schema.column_names:
+                # If it is a list/sparse feature (in tuple representation), uses the offset as shape
+                if isinstance(v, tuple) and isinstance(v[1], tf.TensorShape):
+                    new_input_shapes[k] = tf.TensorShape([v[1][0], None])
+                else:
+                    # Reducing 1 position of the seq length
+                    new_input_shapes[k] = tf.TensorShape([v[0], v[1] - 1])
+
+        return new_input_shapes
+
 
 @Block.registry.register_with_multiple_names("seq_predict_last")
 @tf.keras.utils.register_keras_serializable(package="merlin_models")
@@ -258,6 +269,7 @@ class SequencePredictLast(SequenceTransform):
         so that the tensors sequences can be processed
     """
 
+    @tf.function
     def call(
         self, inputs: TabularData, targets=None, training=False, testing=False, **kwargs
     ) -> Tuple:
@@ -283,6 +295,20 @@ class SequencePredictLast(SequenceTransform):
 
         return (new_inputs, targets)
 
+    def compute_output_shape(self, input_shape):
+        new_input_shapes = dict()
+        for k, v in input_shape.items():
+            new_input_shapes[k] = v
+            if k in self.schema.column_names:
+                # If it is a list/sparse feature (in tuple representation), uses the offset as shape
+                if isinstance(v, tuple) and isinstance(v[1], tf.TensorShape):
+                    new_input_shapes[k] = tf.TensorShape([v[1][0], None])
+                else:
+                    # Reducing 1 position of the seq length
+                    new_input_shapes[k] = tf.TensorShape([v[0], v[1] - 1])
+
+        return new_input_shapes
+
 
 @Block.registry.register_with_multiple_names("seq_predict_random")
 @tf.keras.utils.register_keras_serializable(package="merlin_models")
@@ -374,7 +400,7 @@ class SequenceTargetAsInput(SequenceTransform):
     @tf.function
     def call(
         self, inputs: TabularData, targets=None, training=False, testing=False, **kwargs
-    ) -> Prediction:
+    ) -> Tuple:
         self._check_seq_inputs_targets(inputs)
 
         new_target = tf.identity(inputs[self.target_name])
@@ -385,7 +411,7 @@ class SequenceTargetAsInput(SequenceTransform):
         else:
             raise ValueError("Targets should be None or a dict of tensors")
 
-        return Prediction(inputs, targets)
+        return (inputs, targets)
 
     @classmethod
     def from_config(cls, config):
@@ -454,13 +480,14 @@ class SequenceMaskRandom(SequenceTargetAsInput):
         self.target_mask = self._generate_target_mask(item_id_seq)
 
         inputs_mask = dict()
-        for k, v in inputs.items():
+        for k in inputs:
             if k in self.schema.column_names:
                 inputs_mask[k] = self.target_mask
             else:
                 inputs_mask[k] = None
 
-        return (inputs_mask, self.target_mask)
+        targets_mask = dict({self.target_name: self.target_mask})
+        return (inputs_mask, targets_mask)
 
     def _generate_target_mask(self, ids_seq: tf.RaggedTensor) -> tf.RaggedTensor:
         """Generates a target mask according to the defined probability and
