@@ -66,7 +66,7 @@ class ListToRagged(TabularBlock):
 
 
 @tf.keras.utils.register_keras_serializable(package="merlin.models")
-class ProcessList(TabularBlock):
+class PrepareFeatures(TabularBlock):
     """Process all list (multi-hot/sequential) features.add()
 
     In NVTabular, list-columns are represented as a tuple of (values, offsets).
@@ -84,16 +84,30 @@ class ProcessList(TabularBlock):
 
         for name, val in inputs.items():
             is_ragged = True
-            if name in self.schema:
+            if name in self.schema.column_names:
                 val_count = self.schema[name].properties.get("value_count")
-                if val_count and val_count["min"] == val_count["max"]:
+                if (
+                    val_count
+                    and "min" in val_count
+                    and "max" in val_count
+                    and val_count["min"] == val_count["max"]
+                ):
                     is_ragged = False
 
             if isinstance(val, tuple):
                 ragged = list_col_to_ragged(val)
             elif isinstance(val, tf.SparseTensor):
                 ragged = tf.RaggedTensor.from_sparse(val)
+            elif isinstance(val, tf.RaggedTensor):
+                ragged = val
             else:
+                # Expanding / setting last dim of non-list features to be 1D
+                if (
+                    name in self.schema.column_names
+                    and not self.schema[name].is_list
+                    and not self.schema[name].is_ragged
+                ):
+                    val = tf.reshape(val, (-1, 1))
                 outputs[name] = val
                 continue
 
@@ -114,9 +128,14 @@ class ProcessList(TabularBlock):
             if isinstance(v, tuple) and isinstance(v[1], tf.TensorShape):
                 is_ragged = True
                 max_seq_length = None
-                if k in self.schema:
+                if k in self.schema.column_names:
                     val_count = self.schema[k].properties.get("value_count")
-                    if val_count and val_count["min"] == val_count["max"]:
+                    if (
+                        val_count
+                        and "min" in val_count
+                        and "max" in val_count
+                        and val_count["min"] == val_count["max"]
+                    ):
                         is_ragged = False
                         max_seq_length = val_count["min"]
 
