@@ -93,8 +93,6 @@ def model_test(
     else:
         dataloader = dataset_or_loader
 
-    batch = sample_batch(dataloader, to_ragged=reload_model)
-
     fit_kwargs = fit_kwargs or {}
     losses = model.fit(dataloader, epochs=epochs, steps_per_epoch=1, **fit_kwargs)
 
@@ -105,7 +103,7 @@ def model_test(
 
         assert isinstance(loaded_model, type(model))
 
-        x, y = sample_batch(dataloader, batch_size=50, to_ragged=False, process_lists=False)
+        x, y = sample_batch(dataloader, batch_size=50, to_ragged=False, prepare_features=False)
         batch = [(x, y)]
 
         model_preds = model.predict(iter(batch))
@@ -114,18 +112,30 @@ def model_test(
         if isinstance(model_preds, dict):
             for task_name in model_preds:
                 tf.debugging.assert_near(
-                    model_preds[task_name],
-                    loaded_model_preds[task_name],
+                    model_preds[task_name], loaded_model_preds[task_name], atol=1e-5
                 )
         else:
-            tf.debugging.assert_near(
-                model_preds,
-                loaded_model_preds,
-            )
+            tf.debugging.assert_near(model_preds, loaded_model_preds, atol=1e-5)
 
         loaded_model.compile(run_eagerly=run_eagerly, optimizer=optimizer, **kwargs)
         loaded_model.fit(iter(batch))
 
+        if model.input_schema:
+            signature = loaded_model.signatures["serving_default"]
+            signature_input_names = set(signature.structured_input_signature[1].keys())
+
+            model_input_names = []
+            for col in model.input_schema:
+                if col.is_list:
+                    # list columns are currently always passed
+                    # in as a tuple of (values, row_lengths)
+                    for i in ["1", "2"]:
+                        model_input_names.append(f"{col.name}_{i}")
+                else:
+                    model_input_names.append(col.name)
+
+            assert signature_input_names == set(model_input_names)
+
         return loaded_model, losses
 
     dataloader.stop()
