@@ -142,7 +142,7 @@ def get_output_sizes_from_schema(schema: Schema, batch_size=-1, max_sequence_len
     for feature in schema:
         name = feature.name
         # Sequential or multi-hot feature
-        if feature.is_list:
+        if feature.is_list and feature.value_count.max:
             sizes[name] = torch.Size(
                 [
                     batch_size,
