@@ -1,3 +1,4 @@
+import pytest
 from testbook import testbook
 
 from tests.conftest import REPO_ROOT
@@ -6,6 +7,7 @@ from tests.conftest import REPO_ROOT
 @testbook(
     REPO_ROOT / "examples/usecases/entertainment-with-pretrained-embeddings.ipynb", execute=False
 )
+@pytest.mark.notebook
 def test_usecase_pretrained_embeddings(tb):
     tb.execute()
     history = tb.ref("history")
