@@ -14,6 +14,7 @@
 # limitations under the License.
 #
 import copy
+import pickle
 
 import numpy as np
 import pandas as pd
@@ -22,13 +23,40 @@ import tensorflow as tf
 from tensorflow.test import TestCase
 
 import merlin.models.tf as mm
+from merlin.core.dispatch import make_df
 from merlin.datasets.synthetic import generate_data
 from merlin.io.dataset import Dataset
+from merlin.models.tf.models.base import get_output_schema
 from merlin.models.tf.utils import testing_utils, tf_utils
 from merlin.models.utils import schema_utils
 from merlin.schema import ColumnSchema, Schema, Tags
 
 
+class TestGetOutputSchema:
+    def test_scalar(self, tmpdir):
+        inputs = tf.keras.Input(shape=(1), name="my_input")
+        outputs = tf.keras.layers.Dense(1, name="my_output")(inputs)
+        model = tf.keras.models.Model(inputs=inputs, outputs=outputs)
+        _ = model({"my_input": tf.constant([[1]])})
+        model.save(tmpdir)
+        output_schema = get_output_schema(tmpdir)
+        output_col = output_schema["my_output"]
+        assert output_col.is_list is False
+        assert output_col.is_ragged is False
+
+    def test_fixed_list(self, tmpdir):
+        inputs = tf.keras.Input(shape=(1), name="my_input")
+        outputs = tf.keras.layers.Dense(4, name="my_output")(inputs)
+        model = tf.keras.models.Model(inputs=inputs, outputs=outputs)
+        _ = model({"my_input": tf.constant([[1]])})
+        model.save(tmpdir)
+        output_schema = get_output_schema(tmpdir)
+        output_col = output_schema["my_output"]
+        assert output_col.value_count.min == output_col.value_count.max == 4
+        assert output_col.is_list is True
+        assert output_col.is_ragged is False
+
+
 @pytest.mark.parametrize("run_eagerly", [False])
 def test_simple_model(ecommerce_data: Dataset, run_eagerly):
     model = mm.Model(
@@ -64,12 +92,16 @@ def test_fit_compile_twice():
 
 
 @pytest.mark.parametrize("run_eagerly", [True, False])
-def test_model_from_block(ecommerce_data: Dataset, run_eagerly):
+@pytest.mark.parametrize(
+    "prediction_block",
+    [None, mm.BinaryOutput("click"), mm.BinaryClassificationTask("click")],
+)
+def test_model_from_block(ecommerce_data: Dataset, run_eagerly, prediction_block):
     embedding_options = mm.EmbeddingOptions(embedding_dim_default=2)
     model = mm.Model.from_block(
         mm.MLPBlock([4]),
         ecommerce_data.schema,
-        prediction_tasks=mm.BinaryClassificationTask("click"),
+        prediction_tasks=prediction_block,
         embedding_options=embedding_options,
     )
 
@@ -93,6 +125,49 @@ def test_block_from_model_with_input(ecommerce_data: Dataset):
     assert "The block already includes an InputBlock" in str(excinfo.value)
 
 
+@pytest.mark.parametrize("run_eagerly", [True, False])
+def test_simple_seq_model_with_custom_emb_combiner(sequence_testing_data: Dataset, run_eagerly):
+    schema = sequence_testing_data.schema.select_by_tag(Tags.CATEGORICAL)
+    schema = schema + Schema(
+        [
+            ColumnSchema("item_id_seq_weights", is_list=True),
+            ColumnSchema("click", tags=[Tags.TARGET, Tags.BINARY_CLASSIFICATION]),
+        ]
+    )
+    # Add a weight feature and a binary target
+    data = sequence_testing_data.compute()
+    data["item_id_seq_weights"] = list(np.random.uniform(0, 1, (100, 4)))
+    data["click"] = list(np.random.randint(0, 2, (100,)))
+
+    loader = mm.Loader(Dataset(data, schema=schema), batch_size=100)
+
+    # Define the input block with a weighted average embeddings combiner
+    input_block_weighed_avg_model = mm.InputBlockV2(
+        schema,
+        categorical=mm.Embeddings(
+            schema.select_by_tag(Tags.CATEGORICAL),
+            sequence_combiner=mm.AverageEmbeddingsByWeightFeature.from_schema_convention(
+                schema, "_weights"
+            ),
+        ),
+    )
+
+    model = mm.Model(
+        input_block_weighed_avg_model,
+        mm.MLPBlock([4]),
+        mm.BinaryClassificationTask("click"),
+    )
+
+    testing_utils.model_test(model, loader, run_eagerly=run_eagerly, reload_model=True)
+
+    batch = mm.sample_batch(loader, include_targets=False, to_ragged=True)
+    out = model(batch)
+    assert out.shape == (100, 1)
+
+    predictions = model.predict(loader, batch_size=8, steps=1)
+    assert predictions.shape == (100, 1)
+
+
 class MetricsLogger(tf.keras.callbacks.Callback):
     """Callback to keep track of the metrics returned on each step in every epoch.
 
@@ -145,6 +220,7 @@ class UpdateCountMetric(tf.keras.metrics.Metric):
             self.call_count.assign(tf.constant([0.0]))
 
 
+@pytest.mark.parametrize("run_eagerly", [True, False])
 @pytest.mark.parametrize(
     ["num_rows", "batch_size", "train_metrics_steps", "expected_steps", "expected_metrics_steps"],
     [
@@ -155,7 +231,7 @@ class UpdateCountMetric(tf.keras.metrics.Metric):
     ],
 )
 def test_train_metrics_steps(
-    num_rows, batch_size, train_metrics_steps, expected_steps, expected_metrics_steps
+    run_eagerly, num_rows, batch_size, train_metrics_steps, expected_steps, expected_metrics_steps
 ):
     dataset = generate_data("e-commerce", num_rows=num_rows)
     model = mm.Model(
@@ -164,7 +240,7 @@ def test_train_metrics_steps(
         mm.BinaryClassificationTask("click"),
     )
     model.compile(
-        run_eagerly=True,
+        run_eagerly=run_eagerly,
         optimizer="adam",
         metrics=[UpdateCountMetric()],
     )
@@ -674,7 +750,71 @@ def test_unfreeze_all_blocks(ecommerce_data):
     model.fit(ecommerce_data, batch_size=128, epochs=1)
 
 
-def test_save_and_load(tmpdir):
+class TestModelInputFeatures:
+    def _get_dataset(self):
+        dataset = Dataset(
+            make_df(
+                {
+                    "a": [1, 2],
+                    "b": [3, 4],
+                    "click": [0, 1],
+                }
+            ),
+            schema=Schema(
+                [
+                    ColumnSchema("a", tags=[Tags.CONTINUOUS], dtype=np.float32),
+                    ColumnSchema(
+                        "b",
+                        tags=[Tags.CATEGORICAL],
+                        dtype=np.int32,
+                        properties={"domain": {"min": 0, "max": 10}},
+                    ),
+                    ColumnSchema(
+                        "click",
+                        tags=[Tags.TARGET],
+                        dtype=np.int32,
+                        properties={"domain": {"min": 0, "max": 1}},
+                    ),
+                ]
+            ),
+        )
+        return dataset
+
+    def test_saved_model_input_features(self, tmpdir):
+        dataset = self._get_dataset()
+        input_schema = dataset.schema.select_by_name(["a"])
+        dataset.schema = dataset.schema.select_by_name(["a", "click"])
+        model = mm.Model(
+            mm.InputBlockV2(input_schema),
+            mm.MLPBlock([4]),
+            mm.BinaryClassificationTask("click"),
+        )
+        model.compile()
+        model.fit(dataset, batch_size=2)
+        model.save(tmpdir)
+        reloaded_model = mm.Model.load(tmpdir)
+        signature = reloaded_model.signatures["serving_default"]
+        assert set(signature.structured_input_signature[1]) == {"a"}
+
+    def test_passing_incorrect_features(self):
+        dataset = self._get_dataset()
+        input_schema = dataset.schema.select_by_name(["a"])
+        model = mm.Model(
+            mm.InputBlockV2(input_schema),
+            mm.MLPBlock([4]),
+            mm.BinaryClassificationTask("click"),
+        )
+        model.compile()
+
+        loader = (batch for batch in mm.Loader(dataset, 2))
+
+        with pytest.raises(ValueError) as exc_info:
+            model.fit(loader, batch_size=2)
+
+        assert "Model called with a different set of features" in str(exc_info.value)
+
+
+def test_pickle():
     dataset = generate_data("e-commerce", num_rows=10)
     dataset.schema = dataset.schema.select_by_name(["click", "user_age"])
     model = mm.Model(
@@ -688,6 +828,30 @@ def test_save_and_load(tmpdir):
         epochs=1,
         batch_size=10,
     )
+    pickled = pickle.dumps(model)
+    reloaded_model = pickle.loads(pickled)
+
+    test_case = TestCase()
+    test_case.assertAllClose(
+        model.predict(dataset, batch_size=10), reloaded_model.predict(dataset, batch_size=10)
+    )
+
+
+def test_save_and_load(tmpdir):
+    dataset = generate_data("e-commerce", num_rows=10)
+    input_schema = dataset.schema.select_by_name(["user_age"])
+    dataset.schema = dataset.schema.select_by_name(["user_age", "click"])
+    model = mm.Model(
+        mm.InputBlockV2(input_schema),
+        mm.MLPBlock([4]),
+        mm.BinaryClassificationTask("click"),
+    )
+    model.compile()
+    _ = model.fit(
+        dataset,
+        epochs=1,
+        batch_size=10,
+    )
     model.save(tmpdir)
     reloaded_model = mm.Model.load(tmpdir)
 
