@@ -2,6 +2,7 @@ from typing import Dict, Optional
 
 import pytest
 import tensorflow as tf
+from tensorflow.keras.metrics import Metric
 
 import merlin.models.tf as mm
 from merlin.io import Dataset
@@ -16,17 +17,16 @@ from merlin.models.tf.utils import testing_utils
         None,
         mm.MLPBlock([32]),
         dict(click=mm.MLPBlock([16]), play_percentage=mm.MLPBlock([20])),
-        dict(binary_classification_task=mm.MLPBlock([16]), regression_task=mm.MLPBlock([20])),
         {
             "click/binary_classification_task": mm.MLPBlock([16]),
             "play_percentage/regression_task": mm.MLPBlock([20]),
         },
     ],
 )
-def test_model_with_multiple_tasks(music_streaming_data: Dataset, task_blocks, run_eagerly: bool):
-    music_streaming_data.schema = music_streaming_data.schema.without("like")
-
-    inputs = mm.InputBlock(music_streaming_data.schema)
+def test_model_with_multiple_tasks_with_task_towers(
+    music_streaming_data: Dataset, task_blocks, run_eagerly: bool
+):
+    inputs = mm.InputBlockV2(music_streaming_data.schema)
     prediction_tasks = mm.PredictionTasks(music_streaming_data.schema, task_blocks=task_blocks)
     model = mm.Model(inputs, mm.MLPBlock([64]), prediction_tasks)
     model.compile(optimizer="adam", run_eagerly=run_eagerly)
@@ -40,16 +40,232 @@ def test_model_with_multiple_tasks(music_streaming_data: Dataset, task_blocks, r
             "loss_batch",
             "regularization_loss",
             "click/binary_classification_task_loss",
-            "play_percentage/regression_task_loss",
-            "play_percentage/regression_task_root_mean_squared_error",
             "click/binary_classification_task_precision",
             "click/binary_classification_task_recall",
             "click/binary_classification_task_binary_accuracy",
             "click/binary_classification_task_auc",
+            "like/binary_classification_task_loss",
+            "like/binary_classification_task_precision",
+            "like/binary_classification_task_recall",
+            "like/binary_classification_task_binary_accuracy",
+            "like/binary_classification_task_auc",
+            "play_percentage/regression_task_loss",
+            "play_percentage/regression_task_root_mean_squared_error",
         ]
     )
     if task_blocks:
-        assert model.prediction_tasks[0].task_block != model.prediction_tasks[1].task_block
+        assert model.prediction_tasks[0].task_block != model.prediction_tasks[2].task_block
+        if isinstance(task_blocks, dict):
+            # Ensures for like there is no task tower
+            assert model.prediction_tasks[1].task_block is None
+        else:
+            assert model.prediction_tasks[0].task_block != model.prediction_tasks[1].task_block
+
+
+@pytest.mark.parametrize("run_eagerly", [False])
+@pytest.mark.parametrize(
+    "metrics",
+    [
+        {
+            "click/binary_classification_task": (
+                tf.keras.metrics.Precision(name="precision"),
+                tf.keras.metrics.Recall(name="recall"),
+            ),
+            "like/binary_classification_task": [
+                tf.keras.metrics.BinaryAccuracy(name="binary_accuracy")
+            ],
+        },
+        None,
+        tf.keras.metrics.AUC(name="auc"),
+        (tf.keras.metrics.Precision(name="precision"), tf.keras.metrics.Recall(name="recall")),
+    ],
+)
+def test_model_with_multiple_tasks_metrics(
+    music_streaming_data: Dataset, run_eagerly: bool, metrics
+):
+    music_streaming_data.schema = music_streaming_data.schema.without("play_percentage")
+
+    inputs = mm.InputBlockV2(music_streaming_data.schema)
+    prediction_tasks = mm.PredictionTasks(music_streaming_data.schema)
+    model = mm.Model(inputs, mm.MLPBlock([64]), prediction_tasks)
+
+    weighted_metrics = metrics
+
+    expected_metrics = [
+        "loss",
+        "regularization_loss",
+        "loss_batch",
+        "click/binary_classification_task_loss",
+        "like/binary_classification_task_loss",
+    ]
+    if isinstance(metrics, Metric):
+        expected_metrics.extend(
+            [
+                "click/binary_classification_task_auc",
+                "like/binary_classification_task_auc",
+                "click/binary_classification_task_weighted_auc",
+                "like/binary_classification_task_weighted_auc",
+            ]
+        )
+
+        # Creating another copy of the metrics to avoid error for reusing same metrics
+        # for metrics and weighted_metrics
+        weighted_metrics = (tf.keras.metrics.AUC(name="auc"),)
+    elif isinstance(metrics, (list, tuple)):
+        expected_metrics.extend(
+            [
+                "click/binary_classification_task_precision",
+                "like/binary_classification_task_precision",
+                "click/binary_classification_task_recall",
+                "like/binary_classification_task_recall",
+                "click/binary_classification_task_weighted_precision",
+                "like/binary_classification_task_weighted_precision",
+                "click/binary_classification_task_weighted_recall",
+                "like/binary_classification_task_weighted_recall",
+            ]
+        )
+    elif isinstance(metrics, dict):
+        expected_metrics.extend(
+            [
+                "click/binary_classification_task_precision",
+                "click/binary_classification_task_recall",
+                "like/binary_classification_task_binary_accuracy",
+                "click/binary_classification_task_weighted_precision",
+                "click/binary_classification_task_weighted_recall",
+                "like/binary_classification_task_weighted_binary_accuracy",
+            ]
+        )
+        # Creating another copy of the metrics to avoid error for reusing same metrics
+        # for metrics and weighted_metrics
+        weighted_metrics = {
+            "click/binary_classification_task": (
+                tf.keras.metrics.Precision(name="precision"),
+                tf.keras.metrics.Recall(name="recall"),
+            ),
+            "like/binary_classification_task": [
+                tf.keras.metrics.BinaryAccuracy(name="binary_accuracy")
+            ],
+        }
+    elif metrics is None:
+        # Use default metrics
+        expected_metrics.extend(
+            [
+                "click/binary_classification_task_precision",
+                "click/binary_classification_task_recall",
+                "click/binary_classification_task_binary_accuracy",
+                "click/binary_classification_task_auc",
+                "like/binary_classification_task_precision",
+                "like/binary_classification_task_recall",
+                "like/binary_classification_task_binary_accuracy",
+                "like/binary_classification_task_auc",
+            ]
+        )
+
+    model.compile(
+        optimizer="adam",
+        run_eagerly=run_eagerly,
+        metrics=metrics,
+        weighted_metrics=weighted_metrics,
+        from_serialized=False,
+    )
+
+    metrics_results = model.train_step(mm.sample_batch(music_streaming_data, batch_size=50))
+
+    assert metrics_results["loss"] >= 0
+    assert set(metrics_results.keys()) == set(expected_metrics)
+
+
+@testing_utils.mark_run_eagerly_modes
+def test_model_with_multiple_tasks_loss_weights_and_weighted_metrics(
+    music_streaming_data: Dataset, run_eagerly: bool
+):
+    inputs = mm.InputBlockV2(music_streaming_data.schema)
+    prediction_tasks = mm.PredictionTasks(music_streaming_data.schema)
+    model = mm.Model(inputs, mm.MLPBlock([64]), prediction_tasks)
+
+    loss_weights = {
+        "click/binary_classification_task": 1.0,
+        "play_percentage/regression_task": 2.0,
+        "like/binary_classification_task": 3.0,
+    }
+
+    weighted_metrics = {
+        "click/binary_classification_task": (
+            tf.keras.metrics.Precision(name="precision"),
+            tf.keras.metrics.Recall(name="recall"),
+            tf.keras.metrics.BinaryAccuracy(name="binary_accuracy"),
+            tf.keras.metrics.AUC(name="auc"),
+        ),
+        "like/binary_classification_task": (
+            tf.keras.metrics.Precision(name="weighted_precision"),
+            tf.keras.metrics.Recall(name="weighted_recall"),
+            tf.keras.metrics.BinaryAccuracy(name="binary_accuracy"),
+            tf.keras.metrics.AUC(name="weighted_auc"),
+        ),
+        "play_percentage/regression_task": (
+            tf.keras.metrics.RootMeanSquaredError(name="root_mean_squared_error"),
+        ),
+    }
+
+    model.compile(
+        optimizer="adam",
+        run_eagerly=run_eagerly,
+        loss_weights=loss_weights,
+        weighted_metrics=weighted_metrics,
+    )
+
+    batch = mm.sample_batch(music_streaming_data, batch_size=50)
+
+    metrics = model.test_step(batch)
+
+    assert metrics["loss"] >= 0
+    assert set(metrics.keys()) == set(
+        [
+            "loss",
+            "click/binary_classification_task_loss",
+            "like/binary_classification_task_loss",
+            "play_percentage/regression_task_loss",
+            "click/binary_classification_task_precision",
+            "click/binary_classification_task_recall",
+            "click/binary_classification_task_binary_accuracy",
+            "click/binary_classification_task_auc",
+            "click/binary_classification_task_weighted_precision",
+            "click/binary_classification_task_weighted_recall",
+            "click/binary_classification_task_weighted_binary_accuracy",
+            "click/binary_classification_task_weighted_auc",
+            "like/binary_classification_task_precision",
+            "like/binary_classification_task_recall",
+            "like/binary_classification_task_binary_accuracy",
+            "like/binary_classification_task_auc",
+            "like/binary_classification_task_weighted_precision",
+            "like/binary_classification_task_weighted_recall",
+            "like/binary_classification_task_weighted_binary_accuracy",
+            "like/binary_classification_task_weighted_auc",
+            "play_percentage/regression_task_root_mean_squared_error",
+            "play_percentage/regression_task_weighted_root_mean_squared_error",
+            "regularization_loss",
+            "loss_batch",
+        ]
+    )
+
+    metrics2 = model.test_step(batch)
+    assert metrics["loss"] == metrics2["loss"]
+    for m in metrics:
+        assert metrics[m] == metrics2[m]
+
+    # Disabling losses weights
+    model.compile(loss_weights=None)
+    metrics_non_weighted = model.test_step(batch)
+
+    assert metrics_non_weighted["loss"] != metrics["loss"]
+    assert (
+        metrics_non_weighted["click/binary_classification_task_loss"]
+        == metrics["click/binary_classification_task_loss"]
+    )
+    assert (
+        metrics_non_weighted["like/binary_classification_task_loss"]
+        == metrics["like/binary_classification_task_loss"]
+    )
 
 
 @testing_utils.mark_run_eagerly_modes
@@ -101,10 +317,10 @@ def test_mmoe_model(
         "click/binary_classification_task_recall",
         "click/binary_classification_task_binary_accuracy",
         "click/binary_classification_task_auc",
-        "like/binary_classification_task_precision_1",
-        "like/binary_classification_task_recall_1",
+        "like/binary_classification_task_precision",
+        "like/binary_classification_task_recall",
         "like/binary_classification_task_binary_accuracy",
-        "like/binary_classification_task_auc_1",
+        "like/binary_classification_task_auc",
         "play_percentage/regression_task_root_mean_squared_error",
         "regularization_loss",
     ]
@@ -190,11 +406,11 @@ def test_mmoe_block_task_specific_sample_weight_and_weighted_metrics(
             "click/binary_classification_task_weighted_binary_accuracy",
             "click/binary_classification_task_weighted_precision",
             "click/binary_classification_task_weighted_recall",
-            "like/binary_classification_task_auc_1",
+            "like/binary_classification_task_auc",
             "like/binary_classification_task_binary_accuracy",
             "like/binary_classification_task_loss",
-            "like/binary_classification_task_precision_1",
-            "like/binary_classification_task_recall_1",
+            "like/binary_classification_task_precision",
+            "like/binary_classification_task_recall",
             "like/binary_classification_task_weighted_auc",
             "like/binary_classification_task_weighted_binary_accuracy",
             "like/binary_classification_task_weighted_precision",
@@ -264,10 +480,10 @@ def test_cgc_model(music_streaming_data: Dataset, run_eagerly: bool, task_blocks
             "click/binary_classification_task_recall",
             "click/binary_classification_task_binary_accuracy",
             "click/binary_classification_task_auc",
-            "like/binary_classification_task_precision_1",
-            "like/binary_classification_task_recall_1",
+            "like/binary_classification_task_precision",
+            "like/binary_classification_task_recall",
             "like/binary_classification_task_binary_accuracy",
-            "like/binary_classification_task_auc_1",
+            "like/binary_classification_task_auc",
             "play_percentage/regression_task_root_mean_squared_error",
             "regularization_loss",
         ]
@@ -306,10 +522,10 @@ def test_ple_model(music_streaming_data: Dataset, run_eagerly: bool, task_blocks
             "click/binary_classification_task_recall",
             "click/binary_classification_task_binary_accuracy",
             "click/binary_classification_task_auc",
-            "like/binary_classification_task_precision_1",
-            "like/binary_classification_task_recall_1",
+            "like/binary_classification_task_precision",
+            "like/binary_classification_task_recall",
             "like/binary_classification_task_binary_accuracy",
-            "like/binary_classification_task_auc_1",
+            "like/binary_classification_task_auc",
             "play_percentage/regression_task_root_mean_squared_error",
             "regularization_loss",
             "loss_batch",
