@@ -3,6 +3,7 @@ import itertools
 import numpy as np
 import pytest
 import tensorflow as tf
+from tensorflow.keras.utils import set_random_seed
 from transformers import BertConfig
 
 import merlin.models.tf as mm
@@ -27,6 +28,7 @@ def test_import():
 
 @pytest.mark.parametrize("run_eagerly", [True])
 def test_retrieval_transformer(sequence_testing_data: Dataset, run_eagerly):
+    set_random_seed(42)
 
     sequence_testing_data.schema = sequence_testing_data.schema.select_by_tag(
         Tags.SEQUENCE
@@ -78,7 +80,7 @@ def test_retrieval_transformer(sequence_testing_data: Dataset, run_eagerly):
     assert list(item_embeddings.shape) == [51997, d_model]
     predicitons_2 = np.dot(query_embeddings, item_embeddings.T)
 
-    np.testing.assert_allclose(predictions, predicitons_2, atol=1e-7)
+    np.testing.assert_allclose(predictions, predicitons_2, atol=1e-6)
 
 
 def test_transformer_encoder():
@@ -227,7 +229,7 @@ def classification_loader(sequence_testing_data: Dataset):
         sequence_testing_data,
         batch_size=50,
     ).map(mm.ToTarget(schema, "user_country", one_hot=True))
-    return dataloader, schema
+    return dataloader, dataloader.output_schema
 
 
 @pytest.mark.parametrize("run_eagerly", [True, False])
@@ -236,21 +238,27 @@ def test_transformer_with_causal_language_modeling(sequence_testing_data: Datase
     seq_schema = sequence_testing_data.schema.select_by_tag(Tags.SEQUENCE).select_by_tag(
         Tags.CATEGORICAL
     )
-    target = sequence_testing_data.schema.select_by_tag(Tags.ITEM_ID).column_names[0]
+    target_schema = sequence_testing_data.schema.select_by_tag(Tags.ITEM_ID)
+    target = target_schema.column_names[0]
+
+    sequence_testing_data.schema = seq_schema + target_schema
+
     predict_next = mm.SequencePredictNext(schema=seq_schema, target=target)
-
     loader = Loader(sequence_testing_data, batch_size=8, shuffle=False)
+    model_schema = sequence_testing_data.schema
 
+    transformer_input_dim = 48
     model = mm.Model(
         mm.InputBlockV2(
-            seq_schema,
+            model_schema,
             categorical=mm.Embeddings(
-                seq_schema.select_by_tag(Tags.CATEGORICAL), sequence_combiner=None
+                model_schema.select_by_tag(Tags.CATEGORICAL), sequence_combiner=None
             ),
         ),
-        GPT2Block(d_model=48, n_head=8, n_layer=2),
+        mm.MLPBlock([transformer_input_dim]),
+        GPT2Block(d_model=transformer_input_dim, n_head=8, n_layer=2),
         mm.CategoricalOutput(
-            seq_schema.select_by_name(target), default_loss="categorical_crossentropy"
+            model_schema.select_by_name(target), default_loss="categorical_crossentropy"
         ),
     )
 
@@ -271,12 +279,16 @@ def test_transformer_with_causal_language_modeling(sequence_testing_data: Datase
 @pytest.mark.parametrize("run_eagerly", [True, False])
 def test_transformer_with_masked_language_modeling(sequence_testing_data: Dataset, run_eagerly):
 
-    seq_schema = sequence_testing_data.schema.select_by_tag(Tags.SEQUENCE).select_by_tag(
-        Tags.CATEGORICAL
-    )
-    target = sequence_testing_data.schema.select_by_tag(Tags.ITEM_ID).column_names[0]
+    seq_schema = sequence_testing_data.schema.select_by_tag(Tags.SEQUENCE)
+
+    target_schema = sequence_testing_data.schema.select_by_tag(Tags.ITEM_ID)
+    model_schema = seq_schema + target_schema
+    target = target_schema.column_names[0]
+
+    sequence_testing_data.schema = model_schema
 
     loader = Loader(sequence_testing_data, batch_size=8, shuffle=False)
+    transformer_input_dim = 48
     model = mm.Model(
         mm.InputBlockV2(
             seq_schema,
@@ -284,8 +296,9 @@ def test_transformer_with_masked_language_modeling(sequence_testing_data: Datase
                 seq_schema.select_by_tag(Tags.CATEGORICAL), sequence_combiner=None
             ),
         ),
-        BertBlock(
-            d_model=48,
+        mm.MLPBlock([transformer_input_dim]),
+        XLNetBlock(
+            d_model=transformer_input_dim,
             n_head=8,
             n_layer=2,
             pre=mm.SequentialBlock([mm.SequenceMaskLastInference(), mm.ReplaceMaskedEmbeddings()]),
@@ -324,12 +337,16 @@ def test_transformer_with_masked_language_modeling_check_eval_masked(
     sequence_testing_data: Dataset, run_eagerly
 ):
 
-    seq_schema = sequence_testing_data.schema.select_by_tag(Tags.SEQUENCE).select_by_tag(
-        Tags.CATEGORICAL
-    )
-    target = sequence_testing_data.schema.select_by_tag(Tags.ITEM_ID).column_names[0]
+    seq_schema = sequence_testing_data.schema.select_by_tag(Tags.SEQUENCE)
+
+    target_schema = sequence_testing_data.schema.select_by_tag(Tags.ITEM_ID)
+    target = target_schema.column_names[0]
+    model_schema = seq_schema + target_schema
+    sequence_testing_data.schema = model_schema
 
     loader = Loader(sequence_testing_data, batch_size=8, shuffle=False)
+
+    transformer_input_dim = 48
     model = mm.Model(
         mm.InputBlockV2(
             seq_schema,
@@ -337,8 +354,10 @@ def test_transformer_with_masked_language_modeling_check_eval_masked(
                 seq_schema.select_by_tag(Tags.CATEGORICAL), sequence_combiner=None
             ),
         ),
-        # BertBlock(d_model=48, n_head=8, n_layer=2, pre=mm.ReplaceMaskedEmbeddings()),
-        GPT2Block(d_model=48, n_head=8, n_layer=2, pre=mm.ReplaceMaskedEmbeddings()),
+        mm.MLPBlock([transformer_input_dim]),
+        BertBlock(
+            d_model=transformer_input_dim, n_head=8, n_layer=2, pre=mm.ReplaceMaskedEmbeddings()
+        ),
         mm.CategoricalOutput(
             seq_schema.select_by_name(target),
             default_loss="categorical_crossentropy",
@@ -391,3 +410,57 @@ def test_transformer_with_masked_language_modeling_check_eval_masked(
     # Ensures metrics masking only last positions are different then the ones
     # considering all positions
     assert not _metrics_almost_equal(metrics_all_positions1, metrics_last_positions)
+
+
+@pytest.mark.parametrize("run_eagerly", [True, False])
+def test_transformer_model_with_masking_and_broadcast_to_sequence(
+    sequence_testing_data: Dataset, run_eagerly: bool
+):
+    schema = sequence_testing_data.schema
+    seq_schema = schema.select_by_name(["item_id_seq", "categories", "item_age_days_norm"])
+    context_schema = schema.select_by_name(["user_country", "user_age"])
+    sequence_testing_data.schema = seq_schema + context_schema
+
+    target = schema.select_by_tag(Tags.ITEM_ID).column_names[0]
+    item_id_name = schema.select_by_tag(Tags.ITEM_ID).first.properties["domain"]["name"]
+
+    input_block = mm.InputBlockV2(
+        sequence_testing_data.schema,
+        embeddings=mm.Embeddings(
+            seq_schema.select_by_tag(Tags.CATEGORICAL)
+            + context_schema.select_by_tag(Tags.CATEGORICAL),
+            sequence_combiner=None,
+        ),
+        post=mm.BroadcastToSequence(context_schema, seq_schema),
+    )
+
+    dmodel = 32
+    mlp_block = mm.MLPBlock([128, dmodel], activation="relu")
+
+    dense_block = mm.SequentialBlock(
+        input_block,
+        mlp_block,
+        mm.GPT2Block(
+            d_model=dmodel,
+            n_head=4,
+            n_layer=2,
+            pre=mm.ReplaceMaskedEmbeddings(),
+            post="inference_hidden_state",
+        ),
+    )
+
+    mlp_block2 = mm.MLPBlock([128, dmodel], activation="relu")
+
+    prediction_task = mm.CategoricalOutput(
+        to_call=input_block["categorical"][item_id_name],
+    )
+    model = mm.Model(dense_block, mlp_block2, prediction_task)
+
+    fit_pre = mm.SequenceMaskRandom(schema=seq_schema, target=target, masking_prob=0.3)
+    testing_utils.model_test(
+        model,
+        sequence_testing_data,
+        run_eagerly=run_eagerly,
+        reload_model=False,
+        fit_kwargs={"pre": fit_pre},
+    )
