@@ -749,7 +749,7 @@ class TestBroadcastToSequence(tf.test.TestCase):
                 "s2": tf.constant([[[1, 2], [3, 4]], [[6, 3], [2, 3]]]),
             }
             layer(inputs)
-        assert "sequence features must share the same sequence lengths" in str(exc_info.value)
+        assert "Sequential features must share the same sequence lengths" in str(exc_info.value)
 
     def test_different_sequence_lengths_ragged(self):
         context_schema = Schema([ColumnSchema("c1")])
@@ -762,7 +762,7 @@ class TestBroadcastToSequence(tf.test.TestCase):
                 "s2": tf.ragged.constant([[[1, 2], [3, 4]], [[6, 3], [2, 3]]]),
             }
             layer(inputs)
-        assert "sequence features must share the same sequence lengths" in str(exc_info.value)
+        assert "Sequential features must share the same sequence lengths" in str(exc_info.value)
 
     def test_ragged_and_dense_features(self):
         context_schema = Schema([ColumnSchema("c1")])
@@ -775,7 +775,9 @@ class TestBroadcastToSequence(tf.test.TestCase):
                 "s2": tf.ragged.constant([[[1, 2], [3, 4]], [[6, 3], [2, 3]]]),
             }
             layer(inputs)
-        assert "sequence features must all be ragged or all dense, not both" in str(exc_info.value)
+        assert "Sequential features must all be ragged or all dense, not both" in str(
+            exc_info.value
+        )
 
     def test_mask_propagation(self):
         masking_layer = tf.keras.layers.Masking(mask_value=0)
@@ -815,7 +817,7 @@ class TestBroadcastToSequence(tf.test.TestCase):
         sequence_schema = Schema([ColumnSchema("b")])
 
         broadcast_layer = BroadcastToSequence(context_schema, sequence_schema)
-        model = mm.Model(broadcast_layer)
+        model = mm.Model(broadcast_layer, schema=context_schema + sequence_schema)
         outputs = model(partially_masked_inputs)
 
         self.assertAllEqual(outputs["a"]._keras_mask, tf.ragged.constant([[True], [False, True]]))
@@ -855,7 +857,42 @@ class TestBroadcastToSequence(tf.test.TestCase):
         outputs = broadcast_layer(inputs)
         self.assertAllEqual(outputs["sequence_embedding"].shape, tf.TensorShape([3, None, 2]))
         self.assertAllEqual(outputs["context_a"].shape, tf.TensorShape([3, None, 1]))
-        self.assertAllEqual(outputs["context_b"].shape, tf.TensorShape([3, None]))
+        self.assertAllEqual(outputs["context_b"].shape, tf.TensorShape([3, None, 1]))
+
+
+def test_broadcast_to_sequence_input_block(sequence_testing_data: Dataset):
+    schema = sequence_testing_data.schema
+    seq_schema = schema.select_by_name(["item_id_seq", "categories", "item_age_days_norm"])
+    context_schema = schema.select_by_name(["user_age", "user_country"])
+    sequence_testing_data.schema = seq_schema + context_schema
+
+    input_block = mm.InputBlockV2(
+        sequence_testing_data.schema,
+        embeddings=mm.Embeddings(
+            seq_schema.select_by_tag(Tags.CATEGORICAL)
+            + context_schema.select_by_tag(Tags.CATEGORICAL),
+            sequence_combiner=None,
+        ),
+        post=mm.BroadcastToSequence(context_schema, seq_schema),
+        aggregation=None,
+    )
+
+    batch = mm.sample_batch(
+        sequence_testing_data, batch_size=100, include_targets=False, to_ragged=True
+    )
+    input_batch = input_block(batch)
+    assert set(input_batch.keys()) == set(
+        ["item_id_seq", "categories", "item_age_days_norm", "user_age", "user_country"]
+    )
+    assert set([len(v.shape) for v in input_batch.values()]) == set([3])
+    assert set([tuple(list(v.shape)[:-1]) for v in input_batch.values()]) == set(
+        [tuple([100, None])]
+    )
+    assert list(input_batch["item_id_seq"].shape) == [100, None, 32]
+    assert list(input_batch["categories"].shape) == [100, None, 16]
+    assert list(input_batch["item_age_days_norm"].shape) == [100, None, 1]
+    assert list(input_batch["user_age"].shape) == [100, None, 1]
+    assert list(input_batch["user_country"].shape) == [100, None, 8]
 
 
 @pytest.mark.parametrize(
